package dev.shadowsoffire.apotheosis.adventure.affix;

import net.minecraft.nbt.CompoundTag;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemItem;
import java.util.UUID;
import net.minecraft.resources.ResourceLocation;
import com.mojang.datafixers.kinds.Applicative;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import net.minecraftforge.registries.ForgeRegistries;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.adventure.AdventureModule;
import java.util.function.BiConsumer;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.ChatFormatting;
import dev.shadowsoffire.attributeslib.api.IFormattableAttribute;
import dev.shadowsoffire.attributeslib.AttributesLib;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.world.item.ItemStack;
import java.util.function.Function;
import java.util.stream.Collectors;
import org.apache.commons.lang3.tuple.Pair;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import java.util.Set;
import dev.shadowsoffire.placebo.util.StepFunction;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import java.util.Map;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.entity.ai.attributes.Attribute;
import com.mojang.serialization.Codec;

public class AttributeAffix extends Affix
{
    public static final Codec<AttributeAffix> CODEC;
    protected final Attribute attribute;
    protected final AttributeModifier.Operation operation;
    protected final Map<LootRarity, StepFunction> values;
    protected final Set<LootCategory> types;
    protected final transient Map<LootRarity, ModifierInst> modifiers;
    
    public AttributeAffix(final Attribute attr, final AttributeModifier.Operation op, final Map<LootRarity, StepFunction> values, final Set<LootCategory> types) {
        super(AffixType.STAT);
        this.attribute = attr;
        this.operation = op;
        this.values = values;
        this.types = types;
        this.modifiers = values.entrySet().stream().map(entry -> Pair.of((Object)entry.getKey(), (Object)new ModifierInst(attr, op, entry.getValue()))).collect(Collectors.toMap((Function<? super Object, ? extends LootRarity>)Pair::getKey, (Function<? super Object, ? extends ModifierInst>)Pair::getValue));
    }
    
    @Override
    public MutableComponent getDescription(final ItemStack stack, final LootRarity rarity, final float level) {
        return Component.m_237119_();
    }
    
    @Override
    public Component getAugmentingText(final ItemStack stack, final LootRarity rarity, final float level) {
        final ModifierInst modif = this.modifiers.get(rarity);
        final double value = modif.valueFactory.get(level);
        final MutableComponent valueComp = IFormattableAttribute.toValueComponent(this.attribute, this.operation, (value < 0.0) ? (-value) : value, AttributesLib.getTooltipFlag());
        MutableComponent comp;
        if (value > 0.0) {
            comp = Component.m_237110_("attributeslib.modifier.plus", new Object[] { valueComp, Component.m_237115_(this.attribute.m_22087_()) }).m_130940_(ChatFormatting.BLUE);
        }
        else {
            comp = Component.m_237110_("attributeslib.modifier.take", new Object[] { valueComp, Component.m_237115_(this.attribute.m_22087_()) }).m_130940_(ChatFormatting.RED);
        }
        if (modif.valueFactory.get(0.0f) != modif.valueFactory.get(1.0f)) {
            final Component minComp = (Component)IFormattableAttribute.toValueComponent(this.attribute, this.operation, (double)modif.valueFactory.get(0.0f), AttributesLib.getTooltipFlag());
            final Component maxComp = (Component)IFormattableAttribute.toValueComponent(this.attribute, this.operation, (double)modif.valueFactory.get(1.0f), AttributesLib.getTooltipFlag());
            comp.m_7220_((Component)Affix.valueBounds(minComp, maxComp));
        }
        return (Component)comp;
    }
    
    @Override
    public void addModifiers(final ItemStack stack, final LootRarity rarity, final float level, final EquipmentSlot type, final BiConsumer<Attribute, AttributeModifier> map) {
        final LootCategory cat = LootCategory.forItem(stack);
        if (cat.isNone()) {
            AdventureModule.LOGGER.debug("Attempted to apply the attributes of affix {} on item {}, but it is not an affix-compatible item!", (Object)this.getId(), (Object)stack.m_41786_().getString());
            return;
        }
        final ModifierInst modif = this.modifiers.get(rarity);
        if (modif.attr == null) {
            AdventureModule.LOGGER.debug("The affix {} has attempted to apply a null attribute modifier to {}!", (Object)this.getId(), (Object)stack.m_41786_().getString());
            return;
        }
        for (final EquipmentSlot slot : cat.getSlots()) {
            if (slot == type) {
                map.accept(modif.attr, modif.build(stack, this.getId(), level));
            }
        }
    }
    
    @Override
    public boolean canApplyTo(final ItemStack stack, final LootCategory cat, final LootRarity rarity) {
        return !cat.isNone() && (this.types.isEmpty() || this.types.contains(cat)) && this.modifiers.containsKey(rarity);
    }
    
    public Codec<? extends Affix> getCodec() {
        return AttributeAffix.CODEC;
    }
    
    static {
        CODEC = RecordCodecBuilder.create(inst -> inst.group((App)ForgeRegistries.ATTRIBUTES.getCodec().fieldOf("attribute").forGetter(a -> a.attribute), (App)PlaceboCodecs.enumCodec((Class)AttributeModifier.Operation.class).fieldOf("operation").forGetter(a -> a.operation), (App)GemBonus.VALUES_CODEC.fieldOf("values").forGetter(a -> a.values), (App)LootCategory.SET_CODEC.fieldOf("types").forGetter(a -> a.types)).apply((Applicative)inst, AttributeAffix::new));
    }
    
    record ModifierInst(Attribute attr, AttributeModifier.Operation op, StepFunction valueFactory) {
        private static UUID getOrCreateUUID(final ItemStack stack, final ResourceLocation id) {
            final CompoundTag tag = stack.m_41737_("affix_data");
            return GemItem.getOrCreateUUIDs(tag, 1).get(0);
        }
        
        public AttributeModifier build(final ItemStack stack, final ResourceLocation id, final float level) {
            return new AttributeModifier(getOrCreateUUID(stack, id), "affix:" + id, (double)this.valueFactory.get(level), this.op);
        }
    }
}
